#define mForward	1
#define mBackward	2
#define mTurnRight	4
#define mTurnLeft	8

#define runSpeed	24
#define walkSpeed	8

#define ActionWalk		1
#define ActionRun		2
#define ActionIdle		3
#define ActionFall		4
#define ActionTalk		5


#define WaitingTalk			1
#define Talking				2
#define TalkingOutOfRange	3

#define FallDistance skill28
#define FallVelocity skill29
#define GravityAccel skill30

#define CurrentAction skill31
#define PreviousAction skill32
#define AnimationPercent skill33

#define Running skill34
#define MovementAction skill35

#define Blending skill36
#define BlendingPercent skill37

#define SizeOffset	skill38
#define RadiusSize	skill39

#define TalkingState skill40


////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Handles the gravity	
////////////////////////////////////////////////////////////////////////////////////////////////////////////
function handle_gravity(var typeMovement){
	VECTOR fromVec;
	VECTOR toVec;
	var downMovement;
	vec_set(fromVec, me.x);
	vec_set(toVec, me.x);
	fromVec.z = fromVec.z -my.SizeOffset;
	toVec.z = toVec.z - 5000;
	if(c_trace(fromVec,toVec,IGNORE_ME)>=0){
		if(trace_hit==1 && (normal.z >= ZNormalTolerance && normal.z <1)){
			typeMovement &= ~GLIDE;					
		}													
		my.FallVelocity += my.GravityAccel * time_step;
		downMovement = c_move(me,vector(0,0,-my.FallVelocity),NULL,typeMovement);
		if(downMovement<=0){
			my.FallVelocity  = 0;
			my.FallDistance = 0;
		}
		else
			my.FallDistance += downMovement;
		//Aqu debemos de actualizar el cdigo de cada
		if(my.FallDistance>=80 && normal.z <=ZNormalTolerance) //If we are falling or sliding through a non climbable angle
			my.CurrentAction = ActionFall;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Handles entities movement
////////////////////////////////////////////////////////////////////////////////////////////////////////////
function handle_movement(VECTOR* displace, VECTOR* absMovement, var otherFlags){
	var typeGravity = IGNORE_MODELS | IGNORE_PASSABLE | GLIDE | otherFlags;
	if(vec_length(displace)!=0){
		c_move(me,displace,absMovement,IGNORE_PASSABLE | GLIDE | otherFlags);
		// Check if the angle is too steep
		if(trace_hit==1 && (normal.z >= ZNormalTolerance && normal.z <1)){
			typeGravity &= ~GLIDE;					
		}													
	}
	handle_gravity(typeGravity);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Handles the animations
////////////////////////////////////////////////////////////////////////////////////////////////////////////
function handle_animation() {
	STRING* animationName;
	var animFactor;
	var blendFactor;
	var animStart = 0;
	var animEnd = 100;
	var animType = ANM_CYCLE;
	switch(me.CurrentAction){
		case ActionIdle: 
			animationName = "idle";
			animFactor = 1.5;
			break;
		case ActionRun: 
			animationName = "run";
			animFactor = 7;
			break;
		case ActionWalk: 
			animationName = "walk";
			animFactor = 5;
			break;
		case ActionFall: 
			animationName = "fall";
			animFactor = 15;
			animType = 0;
			break;
	}

	//	Action changed
	if(my.CurrentAction!=my.PreviousAction){
		my.Blending = 1;
		my.BlendingPercent = 0;
	}
	//	Blend
	if(my.Blending){
		if(my.BlendingPercent>=100){					//	Blending ended
			my.Blending = 0;							
			my.BlendingPercent = 0;						
			my.AnimationPercent = animStart;			//	Animation starts in animStart
		} 
		else{
			ent_blend(animationName, animStart, my.BlendingPercent);	//	Blend animation
			my.BlendingPercent += 45 * time_step;			
		}
	}
	//	Animate when not blending
	if(!my.Blending){
		if (my.AnimationPercent > animEnd ){ 							//	Animation ended
			if(animType == ANM_CYCLE)
				my.AnimationPercent = animStart;						//	Restart if ciclic
			else
				my.AnimationPercent = animEnd;
		}
		ent_animate(me,animationName,my.AnimationPercent,animType);
		my.AnimationPercent += animFactor * time_step;
	}
	my.PreviousAction = my.CurrentAction;								//	Update previous action
}
